/*
 * (C)opyright MMIV-MMV Anselm R. Garbe <garbeam at gmail dot com>
 * See LICENSE file for license details.
 */

#ifndef _WMII_H
#define _WMII_H

#include <X11/Xlib.h>

#define MAX_ID              20
#define MAX_BUF             128

#define DEFAULT_FT_ALIGN    "center"
#define DEFAULT_FT_FAM      "monospace"
#define DEFAULT_FT_SIZE     "10"
#define DEFAULT_F_TEXT      "#EEEEEE"
#define DEFAULT_N_TEXT      "#BBBBBB"
#define DEFAULT_F_BG        "#005577"
#define DEFAULT_N_BG        "#222222"
#define DEFAULT_F_BORDER    "#006699 #003366"
#define DEFAULT_N_BORDER    "#444444 #000000"

typedef enum {WEST, NWEST, NORTH, NEAST, EAST,
              SEAST, SOUTH, SWEST, CENTER} Align;

typedef struct Ring Ring;
typedef struct Color Color;
typedef struct Fnt Fnt;
typedef struct Meter Meter;
typedef struct Draw Draw;
typedef struct Action Action;

struct Ring {
    void *item;
    Ring *prev;
    Ring *next;
};

struct Color {
    double r, g, b;
    char colstr[8];
};

struct Fnt {
    Align align;  /* EAST and WEST are valid, CENTER otherwise */
    char *font;
    double scale; /* 10pt is like scale 10 */
};

struct Meter {
    int val;
    Color fill;
};

struct Draw {
    int screen_num;
    Drawable drawable;
    GC gc;
    Color bg;
    Color fg;
    Color border[4];
    XRectangle rect; /* relative rect */
    XRectangle *notch; /* relative notch rect */
    Fnt fnt;
    char *text;
    Meter *meter;
};

struct Action {
    char *name;
    void (*func)(void *obj, char *);
};

/* error.c */
void fatal(char *msg);
int dummy_error_hadler(Display *dpy, XErrorEvent *err);

/* draw.c */
void draw_label(Display *dpy, Draw *r);
void draw_label_noborder(Display *dpy, Draw *r);
void draw_meter(Display *dpy, Draw *r);
int text_width(Display *dpy, char *font,
               double size, unsigned char *utf8txt);

/* expand.c */
/* you have to free return value! */
char *expand_action(char *sockfile, char *action_dir, char *glob,
                    unsigned int depth);

/* ring.c */
Ring *r_attach(Ring *r, void *item);
Ring *r_find(Ring *r, void *pattern, int (*comp)(void *, void *));
Ring *r_top(Ring *r, void *pattern, int (*comp)(void *, void *));
Ring *r_detach(Ring *r, void *item);
void r_swap(Ring *r1, Ring *r2);
void free_ring(Ring *r, void (*free_item)(void *));

/* util.c */
void str_to_align(Align *result, char *val);
char *button_to_str(unsigned int button);
unsigned int str_to_button(char *val);
char *mod_to_str(unsigned long mod);
void str_to_color(Color *c, char *val);
/* convenience function for border color quadrupels */
void str_to_color4(Color c[4], char *val);
void str_to_rect(Display *dpy, XRectangle *root, XRectangle *r, char *val);
unsigned long str_to_mod(char *val);
char *chomp(char *val);
int is_point_in_rect(int x, int y, XRectangle *r);
void setup_wmii_dir();
int count_toks(char *str, char delim);
void tokenize(char **result, const char *str, char delim);
void free_toks(char **toks, int num_toks);
int rect_touches_rect(XRectangle *first, XRectangle *second);
int distance(XRectangle *origin, XRectangle *target);
void spawn_expanded(Display *dpy, char *ds,
                    char *actlocation, char *cmd);
void spawn(void *dpy, char *ds, char *cmd);
/* returns after spawned process exits by itself */
void spawn_wait(void *dpy, char *ds, char *cmd);
void swap(void **p1, void **p2);

/* wm.c */
int property(Display *dpy, Window w, Atom a, Atom t, long l,
             unsigned char **prop);
void install_cursor(Display *dpy, Window w, Cursor c);
void win_prop(Display *dpy, Window w, Atom a, char *res, int len);
void send_message(Display *dpy, Window w, Atom a, long value);
char *display_string(Display *dpy, int screen_num);

#endif /* _WMII_H */
